@@ -1,10 +1,7 @@
 import unittest
-import locale
-import re
 import subprocess
 import sys
 import os
-import warnings
 from test import support
 from test.support import import_helper
 from test.support import os_helper
@@ -23,22 +20,6 @@ except ImportError:
 
 tcl_version = tuple(map(int, _tkinter.TCL_VERSION.split('.')))
 
-_tk_patchlevel = None
-def get_tk_patchlevel():
-    global _tk_patchlevel
-    if _tk_patchlevel is None:
-        tcl = Tcl()
-        patchlevel = tcl.call('info', 'patchlevel')
-        m = re.fullmatch(r'(\d+)\.(\d+)([ab.])(\d+)', patchlevel)
-        major, minor, releaselevel, serial = m.groups()
-        major, minor, serial = int(major), int(minor), int(serial)
-        releaselevel = {'a': 'alpha', 'b': 'beta', '.': 'final'}[releaselevel]
-        if releaselevel == 'final':
-            _tk_patchlevel = major, minor, serial, releaselevel, 0
-        else:
-            _tk_patchlevel = major, minor, 0, releaselevel, serial
-    return _tk_patchlevel
-
 
 class TkinterTest(unittest.TestCase):
 
@@ -70,7 +51,7 @@ class TclTest(unittest.TestCase):
 
     def test_eval_surrogates_in_result(self):
         tcl = self.interp
-        self.assertIn(tcl.eval(r'set a "<\ud83d\udcbb>"'), '<\U0001f4bb>')
+        self.assertEqual(tcl.eval(r'set a "<\ud83d\udcbb>"'), '<\U0001f4bb>')
 
     def testEvalException(self):
         tcl = self.interp
@@ -80,11 +61,30 @@ class TclTest(unittest.TestCase):
         tcl = self.interp
         self.assertRaises(TclError,tcl.eval,'this is wrong')
 
+    def test_eval_returns_tcl_obj(self):
+        tcl = self.interp.tk
+        tcl.eval(r'set a "\u20ac \ud83d\udcbb \0 \udcab"; regexp -about $a')
+        a = tcl.eval('set a')
+        expected = '\u20ac \U0001f4bb \0 \udced\udcb2\udcab'
+        self.assertEqual(a, expected)
+
     def testCall(self):
         tcl = self.interp
         tcl.call('set','a','1')
         self.assertEqual(tcl.call('set','a'),'1')
 
+    def test_call_passing_null(self):
+        tcl = self.interp
+        tcl.call('set', 'a', 'a\0b')  # ASCII-only
+        self.assertEqual(tcl.getvar('a'), 'a\x00b')
+        self.assertEqual(tcl.call('set', 'a'), 'a\x00b')
+        self.assertEqual(tcl.eval('set a'), 'a\x00b')
+
+        tcl.call('set', 'a', '\u20ac\0')  # non-ASCII
+        self.assertEqual(tcl.getvar('a'), '\u20ac\x00')
+        self.assertEqual(tcl.call('set', 'a'), '\u20ac\x00')
+        self.assertEqual(tcl.eval('set a'), '\u20ac\x00')
+
     def testCallException(self):
         tcl = self.interp
         self.assertRaises(TclError,tcl.call,'set','a')
@@ -93,11 +93,35 @@ class TclTest(unittest.TestCase):
         tcl = self.interp
         self.assertRaises(TclError,tcl.call,'this','is','wrong')
 
+    def test_call_returns_tcl_obj(self):
+        tcl = self.interp.tk
+        tcl.eval(r'set a "\u20ac \ud83d\udcbb \0 \udcab"; regexp -about $a')
+        a = tcl.call('set', 'a')
+        expected = '\u20ac \U0001f4bb \0 \udced\udcb2\udcab'
+        if self.wantobjects:
+            self.assertEqual(str(a), expected)
+            self.assertEqual(a.string, expected)
+            self.assertEqual(a.typename, 'regexp')
+        else:
+            self.assertEqual(a, expected)
+
     def testSetVar(self):
         tcl = self.interp
         tcl.setvar('a','1')
         self.assertEqual(tcl.eval('set a'),'1')
 
+    def test_setvar_passing_null(self):
+        tcl = self.interp
+        tcl.setvar('a', 'a\0b')  # ASCII-only
+        self.assertEqual(tcl.getvar('a'), 'a\x00b')
+        self.assertEqual(tcl.call('set', 'a'), 'a\x00b')
+        self.assertEqual(tcl.eval('set a'), 'a\x00b')
+
+        tcl.setvar('a', '\u20ac\0')  # non-ASCII
+        self.assertEqual(tcl.getvar('a'), '\u20ac\x00')
+        self.assertEqual(tcl.call('set', 'a'), '\u20ac\x00')
+        self.assertEqual(tcl.eval('set a'), '\u20ac\x00')
+
     def testSetVarArray(self):
         tcl = self.interp
         tcl.setvar('a(1)','1')
@@ -121,6 +145,18 @@ class TclTest(unittest.TestCase):
         tcl = self.interp
         self.assertRaises(TclError,tcl.getvar,'a(1)')
 
+    def test_getvar_returns_tcl_obj(self):
+        tcl = self.interp.tk
+        tcl.eval(r'set a "\u20ac \ud83d\udcbb \0 \udcab"; regexp -about $a')
+        a = tcl.getvar('a')
+        expected = '\u20ac \U0001f4bb \0 \udced\udcb2\udcab'
+        if self.wantobjects:
+            self.assertEqual(str(a), expected)
+            self.assertEqual(a.string, expected)
+            self.assertEqual(a.typename, 'regexp')
+        else:
+            self.assertEqual(a, expected)
+
     def testUnsetVar(self):
         tcl = self.interp
         tcl.setvar('a',1)
@@ -143,27 +179,21 @@ class TclTest(unittest.TestCase):
         self.assertRaises(TclError,tcl.unsetvar,'a')
 
     def get_integers(self):
-        integers = (0, 1, -1, 2**31-1, -2**31, 2**31, -2**31-1, 2**63-1, -2**63)
-        # bignum was added in Tcl 8.5, but its support is able only since 8.5.8.
-        # Actually it is determined at compile time, so using get_tk_patchlevel()
-        # is not reliable.
-        # TODO: expose full static version.
-        if tcl_version >= (8, 5):
-            v = get_tk_patchlevel()
-            if v >= (8, 6, 0, 'final') or (8, 5, 8) <= v < (8, 6):
-                integers += (2**63, -2**63-1, 2**1000, -2**1000)
-        return integers
+        return (0, 1, -1,
+                2**31-1, -2**31, 2**31, -2**31-1,
+                2**63-1, -2**63, 2**63, -2**63-1,
+                2**1000, -2**1000)
 
     def test_getint(self):
         tcl = self.interp.tk
         for i in self.get_integers():
             self.assertEqual(tcl.getint(' %d ' % i), i)
-            if tcl_version >= (8, 5):
-                self.assertEqual(tcl.getint(' %#o ' % i), i)
-            self.assertEqual(tcl.getint((' %#o ' % i).replace('o', '')), i)
+            self.assertEqual(tcl.getint(' %#o ' % i), i)
+            # Numbers starting with 0 are parsed as decimal in Tcl 9.0
+            # and as octal in older versions.
+            self.assertEqual(tcl.getint((' %#o ' % i).replace('o', '')),
+                             i if tcl_version < (9, 0) else int('%o' % i))
             self.assertEqual(tcl.getint(' %#x ' % i), i)
-        if tcl_version < (8, 5):  # bignum was added in Tcl 8.5
-            self.assertRaises(TclError, tcl.getint, str(2**1000))
         self.assertEqual(tcl.getint(42), 42)
         self.assertRaises(TypeError, tcl.getint)
         self.assertRaises(TypeError, tcl.getint, '42', '10')
@@ -244,10 +274,18 @@ class TclTest(unittest.TestCase):
         with open(filename, 'wb') as f:
             f.write(b"""
             set a "<\xed\xa0\xbd\xed\xb2\xbb>"
+            """)
+        if tcl_version >= (9, 0):
+            self.assertRaises(TclError, tcl.evalfile, filename)
+        else:
+            tcl.evalfile(filename)
+            self.assertEqual(tcl.eval('set a'), '<\U0001f4bb>')
+
+        with open(filename, 'wb') as f:
+            f.write(b"""
             set b "<\\ud83d\\udcbb>"
             """)
         tcl.evalfile(filename)
-        self.assertEqual(tcl.eval('set a'), '<\U0001f4bb>')
         self.assertEqual(tcl.eval('set b'), '<\U0001f4bb>')
 
     def testEvalFileException(self):
@@ -317,8 +355,7 @@ class TclTest(unittest.TestCase):
         check('"a\xbd\u20ac"', 'a\xbd\u20ac')
         check(r'"a\xbd\u20ac"', 'a\xbd\u20ac')
         check(r'"a\0b"', 'a\x00b')
-        if tcl_version >= (8, 5):  # bignum was added in Tcl 8.5
-            check('2**64', str(2**64))
+        check('2**64', str(2**64))
 
     def test_exprdouble(self):
         tcl = self.interp
@@ -349,8 +386,7 @@ class TclTest(unittest.TestCase):
         check('[string length "a\xbd\u20ac"]', 3.0)
         check(r'[string length "a\xbd\u20ac"]', 3.0)
         self.assertRaises(TclError, tcl.exprdouble, '"abc"')
-        if tcl_version >= (8, 5):  # bignum was added in Tcl 8.5
-            check('2**64', float(2**64))
+        check('2**64', float(2**64))
 
     def test_exprlong(self):
         tcl = self.interp
@@ -381,8 +417,7 @@ class TclTest(unittest.TestCase):
         check('[string length "a\xbd\u20ac"]', 3)
         check(r'[string length "a\xbd\u20ac"]', 3)
         self.assertRaises(TclError, tcl.exprlong, '"abc"')
-        if tcl_version >= (8, 5):  # bignum was added in Tcl 8.5
-            self.assertRaises(TclError, tcl.exprlong, '2**64')
+        self.assertRaises(TclError, tcl.exprlong, '2**64')
 
     def test_exprboolean(self):
         tcl = self.interp
@@ -422,10 +457,8 @@ class TclTest(unittest.TestCase):
         check('[string length "a\xbd\u20ac"]', True)
         check(r'[string length "a\xbd\u20ac"]', True)
         self.assertRaises(TclError, tcl.exprboolean, '"abc"')
-        if tcl_version >= (8, 5):  # bignum was added in Tcl 8.5
-            check('2**64', True)
+        check('2**64', True)
 
-    @unittest.skipUnless(tcl_version >= (8, 5), 'requires Tcl version >= 8.5')
     def test_booleans(self):
         tcl = self.interp
         def check(expr, expected):
@@ -455,8 +488,6 @@ class TclTest(unittest.TestCase):
             else:
                 self.assertEqual(result, str(i))
                 self.assertIsInstance(result, str)
-        if get_tk_patchlevel() < (8, 5):  # bignum was added in Tcl 8.5
-            self.assertRaises(TclError, tcl.call, 'expr', str(2**1000))
 
     def test_passing_values(self):
         def passValue(value):
@@ -485,8 +516,6 @@ class TclTest(unittest.TestCase):
                          b'str\xbding' if self.wantobjects else 'str\xbding')
         for i in self.get_integers():
             self.assertEqual(passValue(i), i if self.wantobjects else str(i))
-        if tcl_version < (8, 5):  # bignum was added in Tcl 8.5
-            self.assertEqual(passValue(2**1000), str(2**1000))
         for f in (0.0, 1.0, -1.0, 1/3,
                   sys.float_info.min, sys.float_info.max,
                   -sys.float_info.min, -sys.float_info.max):
@@ -516,29 +545,36 @@ class TclTest(unittest.TestCase):
             return arg
         self.interp.createcommand('testfunc', testfunc)
         self.addCleanup(self.interp.tk.deletecommand, 'testfunc')
-        def check(value, expected=None, *, eq=self.assertEqual):
-            if expected is None:
-                expected = value
+        def check(value, expected1=None, expected2=None, *, eq=self.assertEqual):
+            expected = value
+            if self.wantobjects >= 2:
+                if expected2 is not None:
+                    expected = expected2
+                expected_type = type(expected)
+            else:
+                if expected1 is not None:
+                    expected = expected1
+                expected_type = str
             nonlocal result
             result = None
             r = self.interp.call('testfunc', value)
-            self.assertIsInstance(result, str)
+            self.assertIsInstance(result, expected_type)
             eq(result, expected)
-            self.assertIsInstance(r, str)
+            self.assertIsInstance(r, expected_type)
             eq(r, expected)
         def float_eq(actual, expected):
             self.assertAlmostEqual(float(actual), expected,
                                    delta=abs(expected) * 1e-10)
 
-        check(True, '1')
-        check(False, '0')
+        check(True, '1', 1)
+        check(False, '0', 0)
         check('string')
         check('string\xbd')
         check('string\u20ac')
         check('string\U0001f4bb')
         if sys.platform != 'win32':
-            check('<\udce2\udc82\udcac>', '<\u20ac>')
-            check('<\udced\udca0\udcbd\udced\udcb2\udcbb>', '<\U0001f4bb>')
+            check('<\udce2\udc82\udcac>', '<\u20ac>', '<\u20ac>')
+            check('<\udced\udca0\udcbd\udced\udcb2\udcbb>', '<\U0001f4bb>', '<\U0001f4bb>')
         check('')
         check(b'string', 'string')
         check(b'string\xe2\x82\xac', 'string\xe2\x82\xac')
@@ -552,8 +588,6 @@ class TclTest(unittest.TestCase):
         check(b'str\xc0\x80ing\xe2\x82\xac', 'str\xc0\x80ing\xe2\x82\xac')
         for i in self.get_integers():
             check(i, str(i))
-        if tcl_version < (8, 5):  # bignum was added in Tcl 8.5
-            check(2**1000, str(2**1000))
         for f in (0.0, 1.0, -1.0):
             check(f, repr(f))
         for f in (1/3.0, sys.float_info.min, sys.float_info.max,
@@ -562,9 +596,31 @@ class TclTest(unittest.TestCase):
         check(float('inf'), eq=float_eq)
         check(-float('inf'), eq=float_eq)
         # XXX NaN representation can be not parsable by float()
-        check((), '')
-        check((1, (2,), (3, 4), '5 6', ()), '1 2 {3 4} {5 6} {}')
-        check([1, [2,], [3, 4], '5 6', []], '1 2 {3 4} {5 6} {}')
+        check((), '', '')
+        check((1, (2,), (3, 4), '5 6', ()),
+              '1 2 {3 4} {5 6} {}',
+              (1, (2,), (3, 4), '5 6', ''))
+        check([1, [2,], [3, 4], '5 6', []],
+              '1 2 {3 4} {5 6} {}',
+              (1, (2,), (3, 4), '5 6', ''))
+
+    def test_passing_tcl_obj(self):
+        tcl = self.interp.tk
+        a = None
+        def testfunc(arg):
+            nonlocal a
+            a = arg
+        self.interp.createcommand('testfunc', testfunc)
+        self.addCleanup(self.interp.tk.deletecommand, 'testfunc')
+        tcl.eval(r'set a "\u20ac \ud83d\udcbb \0 \udcab"; regexp -about $a')
+        tcl.eval(r'testfunc $a')
+        expected = '\u20ac \U0001f4bb \0 \udced\udcb2\udcab'
+        if self.wantobjects >= 2:
+            self.assertEqual(str(a), expected)
+            self.assertEqual(a.string, expected)
+            self.assertEqual(a.typename, 'regexp')
+        else:
+            self.assertEqual(a, expected)
 
     def test_splitlist(self):
         splitlist = self.interp.tk.splitlist
@@ -599,77 +655,21 @@ class TclTest(unittest.TestCase):
                 (1, '2', (3.4,)) if self.wantobjects else
                 ('1', '2', '3.4')),
         ]
-        tk_patchlevel = get_tk_patchlevel()
-        if tcl_version >= (8, 5):
-            if not self.wantobjects or tk_patchlevel < (8, 5, 5):
-                # Before 8.5.5 dicts were converted to lists through string
-                expected = ('12', '\u20ac', '\xe2\x82\xac', '3.4')
-            else:
-                expected = (12, '\u20ac', b'\xe2\x82\xac', (3.4,))
-            testcases += [
-                (call('dict', 'create', 12, '\u20ac', b'\xe2\x82\xac', (3.4,)),
-                    expected),
-            ]
-        dbg_info = ('want objects? %s, Tcl version: %s, Tk patchlevel: %s'
-                    % (self.wantobjects, tcl_version, tk_patchlevel))
+        if not self.wantobjects:
+            expected = ('12', '\u20ac', '\xe2\x82\xac', '3.4')
+        else:
+            expected = (12, '\u20ac', b'\xe2\x82\xac', (3.4,))
+        testcases += [
+            (call('dict', 'create', 12, '\u20ac', b'\xe2\x82\xac', (3.4,)),
+                expected),
+        ]
+        dbg_info = ('want objects? %s, Tcl version: %s, Tcl patchlevel: %s'
+                    % (self.wantobjects, tcl_version, self.interp.info_patchlevel()))
         for arg, res in testcases:
             self.assertEqual(splitlist(arg), res,
                              'arg=%a, %s' % (arg, dbg_info))
         self.assertRaises(TclError, splitlist, '{')
 
-    def test_split(self):
-        split = self.interp.tk.split
-        call = self.interp.tk.call
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', r'\bsplit\b.*\bsplitlist\b',
-                                    DeprecationWarning)
-            self.assertRaises(TypeError, split)
-            self.assertRaises(TypeError, split, 'a', 'b')
-            self.assertRaises(TypeError, split, 2)
-        testcases = [
-            ('2', '2'),
-            ('', ''),
-            ('{}', ''),
-            ('""', ''),
-            ('{', '{'),
-            ('a\n b\t\r c\n ', ('a', 'b', 'c')),
-            (b'a\n b\t\r c\n ', ('a', 'b', 'c')),
-            ('a \u20ac', ('a', '\u20ac')),
-            (b'a \xe2\x82\xac', ('a', '\u20ac')),
-            (b'a\xc0\x80b', 'a\x00b'),
-            (b'a\xc0\x80b c\xc0\x80d', ('a\x00b', 'c\x00d')),
-            (b'{a\xc0\x80b c\xc0\x80d', '{a\x00b c\x00d'),
-            ('a {b c}', ('a', ('b', 'c'))),
-            (r'a b\ c', ('a', ('b', 'c'))),
-            (('a', b'b c'), ('a', ('b', 'c'))),
-            (('a', 'b c'), ('a', ('b', 'c'))),
-            ('a 2', ('a', '2')),
-            (('a', 2), ('a', 2)),
-            ('a 3.4', ('a', '3.4')),
-            (('a', 3.4), ('a', 3.4)),
-            (('a', (2, 3.4)), ('a', (2, 3.4))),
-            ((), ()),
-            ([], ()),
-            (['a', 'b c'], ('a', ('b', 'c'))),
-            (['a', ['b', 'c']], ('a', ('b', 'c'))),
-            (call('list', 1, '2', (3.4,)),
-                (1, '2', (3.4,)) if self.wantobjects else
-                ('1', '2', '3.4')),
-        ]
-        if tcl_version >= (8, 5):
-            if not self.wantobjects or get_tk_patchlevel() < (8, 5, 5):
-                # Before 8.5.5 dicts were converted to lists through string
-                expected = ('12', '\u20ac', '\xe2\x82\xac', '3.4')
-            else:
-                expected = (12, '\u20ac', b'\xe2\x82\xac', (3.4,))
-            testcases += [
-                (call('dict', 'create', 12, '\u20ac', b'\xe2\x82\xac', (3.4,)),
-                    expected),
-            ]
-        for arg, res in testcases:
-            with self.assertWarns(DeprecationWarning):
-                self.assertEqual(split(arg), res, msg=arg)
-
     def test_splitdict(self):
         splitdict = tkinter._splitdict
         tcl = self.interp.tk
@@ -695,15 +695,13 @@ class TclTest(unittest.TestCase):
             {'a': (1, 2, 3) if self.wantobjects else '1 2 3',
              'something': 'foo', 'status': ''})
 
-        if tcl_version >= (8, 5):
-            arg = tcl.call('dict', 'create',
-                           '-a', (1, 2, 3), '-something', 'foo', 'status', ())
-            if not self.wantobjects or get_tk_patchlevel() < (8, 5, 5):
-                # Before 8.5.5 dicts were converted to lists through string
-                expected = {'a': '1 2 3', 'something': 'foo', 'status': ''}
-            else:
-                expected = {'a': (1, 2, 3), 'something': 'foo', 'status': ''}
-            self.assertEqual(splitdict(tcl, arg), expected)
+        arg = tcl.call('dict', 'create',
+                       '-a', (1, 2, 3), '-something', 'foo', 'status', ())
+        if not self.wantobjects:
+            expected = {'a': '1 2 3', 'something': 'foo', 'status': ''}
+        else:
+            expected = {'a': (1, 2, 3), 'something': 'foo', 'status': ''}
+        self.assertEqual(splitdict(tcl, arg), expected)
 
     def test_join(self):
         join = tkinter._join
@@ -748,6 +746,7 @@ class TclTest(unittest.TestCase):
         support.check_disallow_instantiation(self, _tkinter.TkttType)
         support.check_disallow_instantiation(self, _tkinter.TkappType)
 
+
 class BigmemTclTest(unittest.TestCase):
 
     def setUp(self):
@@ -782,7 +781,6 @@ class BigmemTclTest(unittest.TestCase):
         self.assertRaises(OverflowError, tk.exprlong, value)
         self.assertRaises(OverflowError, tk.exprboolean, value)
         self.assertRaises(OverflowError, tk.splitlist, value)
-        self.assertRaises(OverflowError, tk.split, value)
         self.assertRaises(OverflowError, tk.createcommand, value, max)
         self.assertRaises(OverflowError, tk.deletecommand, value)
 
@@ -801,7 +799,7 @@ class BigmemTclTest(unittest.TestCase):
 def setUpModule():
     if support.verbose:
         tcl = Tcl()
-        print('patchlevel =', tcl.call('info', 'patchlevel'))
+        print('patchlevel =', tcl.call('info', 'patchlevel'), flush=True)
 
 
 if __name__ == "__main__":
